﻿using Microsoft.AspNetCore.Mvc;
using VeterinaryContracts.BindingModels;
using VeterinaryContracts.BusinessLogicContracts;
using VeterinaryContracts.SearchModels;
using VeterinaryContracts.ViewModels;
using VeterinaryDatabaseImplement.Models;

namespace VeterinaryRestApi.Controllers
{
	[Route("api/[controller]/[action]")]
	[ApiController]
	public class DrugController : Controller
	{
		private readonly ILogger _logger;
		private readonly IDrugLogic _drug;
		public DrugController(ILogger<DrugController> logger, IDrugLogic drug)
		{
			_logger = logger;
			_drug = drug;
		}
		// naher ya eto pisal
		[HttpGet]
		public Tuple<DrugViewModel, List<string>>? GetDrug(int drugId)
		{
			try
			{
				var elem = _drug.ReadElement(new DrugSearchModel { Id = drugId });
				if (elem == null)
					return null;
				return Tuple.Create(elem, elem.DrugMedications.Select(x => x.Value.MedicationName).ToList());
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка получения услуги по id={Id}", drugId);
				throw;
			}
		}
		[HttpGet]
		public DrugViewModel? GetOneDrug(int drugId)
		{
			try
			{
				return _drug.ReadElement(new DrugSearchModel
				{
					Id =drugId
				});
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка получения лекарства по id={Id}",drugId);
				throw;
			}
		}

		[HttpGet]
		public List<DrugViewModel> GetDrugs(int doctorId)
		{
			try
			{
				return _drug.ReadList(new DrugSearchModel { DoctorId = doctorId });
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка получения списка лекарств");
				throw;
			}
		}
		[HttpGet]
		public List<DrugViewModel> GetAllDrugs(int doctorId)
		{
			try
			{
				return _drug.ReadList(null);
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка получения списка лекартсв");
				throw;
			}
		}
		[HttpPost]
		public bool CreateDrug(DrugBindingModel model)
		{
			try
			{
				return _drug.Create(model);
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Не удалось создать лекарств");
				throw;
			}
		}

		[HttpPost]
		public bool UpdateDrug(DrugBindingModel model)
		{
			try
			{
				model.DrugMedications = null!;
				return _drug.Update(model);
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Не удалось обновить лекарства");
				throw;
			}
		}

		[HttpPost]
		public bool DeleteDrug(DrugBindingModel model)
		{
			try
			{
				return _drug.Delete(model);
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка удаления лекарства");
				throw;
			}
		}
	}
}
